@@ -4,21 +4,27 @@ from tests import unittest
 
 from jmespath import parser
 from jmespath import ast
-from jmespath import lexer
-from jmespath import compat
+from jmespath import exceptions
 
 
 class TestParser(unittest.TestCase):
     def setUp(self):
         self.parser = parser.Parser()
 
+    def assert_parsed_ast(self, expression, expected_ast):
+        parsed = self.parser.parse(expression)
+        self.assertEqual(parsed.parsed, expected_ast)
+
+    def test_parse_empty_string_raises_exception(self):
+        with self.assertRaises(exceptions.EmptyExpressionError):
+            self.parser.parse('')
+
     def test_field(self):
-        parsed = self.parser.parse('foo')
-        self.assertEqual(parsed.search({'foo': 'bar'}), 'bar')
+        self.assert_parsed_ast('foo', ast.field('foo'))
 
     def test_dot_syntax(self):
-        parsed = self.parser.parse('foo.bar')
-        self.assertEqual(parsed.search({'foo': {'bar': 'baz'}}), 'baz')
+        self.assert_parsed_ast('foo.bar', ast.sub_expression(ast.field('foo'),
+                                                             ast.field('bar')))
 
     def test_multiple_dots(self):
         parsed = self.parser.parse('foo.bar.baz')
@@ -32,10 +38,8 @@ class TestParser(unittest.TestCase):
             'one')
 
     def test_quoted_subexpression(self):
-        parsed = self.parser.parse('"foo"."bar"')
-        self.assertIsInstance(parsed.parsed, ast.SubExpression)
-        self.assertEqual(parsed.parsed.parent.name, 'foo')
-        self.assertEqual(parsed.parsed.child.name, 'bar')
+        self.assert_parsed_ast('"foo"."bar"', ast.sub_expression(
+            ast.field('foo'), ast.field('bar')))
 
     def test_wildcard(self):
         parsed = self.parser.parse('foo[*]')
@@ -63,19 +67,11 @@ class TestParser(unittest.TestCase):
         self.assertEqual(parsed.search({'foo': {'baz': 'baz'}}), None)
 
     def test_or_repr(self):
-        parsed = self.parser.parse('foo || bar')
-        self.assertEqual(repr(parsed), 'ORExpression(Field(foo), Field(bar))')
+        self.assert_parsed_ast('foo || bar', ast.or_expression(ast.field('foo'),
+                                                               ast.field('bar')))
 
     def test_unicode_literals_escaped(self):
-        parsed = self.parser.parse(r'`"\u2713"`')
-        if compat.PY2:
-            self.assertEqual(repr(parsed), r'Literal(\u2713)')
-        else:
-            self.assertEqual(repr(parsed), u'Literal(\u2713)')
-
-    def test_unicode_pretty_print(self):
-        parsed = self.parser.parse(r'`"\u2713"`')
-        self.assertEqual(parsed.pretty_print(), u'Literal(\u2713)')
+        self.assert_parsed_ast(r'`"\u2713"`', ast.literal(u'\u2713'))
 
     def test_multiselect(self):
         parsed = self.parser.parse('foo.{bar: bar,baz: baz}')
@@ -85,7 +81,6 @@ class TestParser(unittest.TestCase):
 
     def test_multiselect_subexpressions(self):
         parsed = self.parser.parse('foo.{"bar.baz": bar.baz, qux: qux}')
-        foo = parsed.search({'foo': {'bar': {'baz': 'CORRECT'}, 'qux': 'qux'}})
         self.assertEqual(
             parsed.search({'foo': {'bar': {'baz': 'CORRECT'}, 'qux': 'qux'}}),
             {'bar.baz': 'CORRECT', 'qux': 'qux'})
@@ -102,7 +97,7 @@ class TestErrorMessages(unittest.TestCase):
         self.parser = parser.Parser()
 
     def assert_error_message(self, expression, error_message,
-                             exception=parser.ParseError):
+                             exception=exceptions.ParseError):
         try:
             self.parser.parse(expression)
         except exception as e:
@@ -117,12 +112,12 @@ class TestErrorMessages(unittest.TestCase):
                 "ParseError not raised for bad expression: %s" % expression)
 
     def test_bad_parse(self):
-        with self.assertRaises(parser.ParseError):
-            parsed = self.parser.parse('foo]baz')
+        with self.assertRaises(exceptions.ParseError):
+            self.parser.parse('foo]baz')
 
     def test_bad_parse_error_message(self):
         error_message = (
-            'Invalid jmespath expression: Parse error at column 3 '
+            'Unexpected token: ]: Parse error at column 3 '
             'near token "]" (RBRACKET) for expression:\n'
             '"foo]baz"\n'
             '    ^')
@@ -137,25 +132,25 @@ class TestErrorMessages(unittest.TestCase):
 
     def test_bad_lexer_values(self):
         error_message = (
-            'Bad jmespath expression: Bad token \'"bar\': '
-            'starting quote is missing the ending quote:\n'
+            'Bad jmespath expression: '
+            'Starting quote is missing the ending quote:\n'
             'foo."bar\n'
             '    ^')
         self.assert_error_message('foo."bar', error_message,
-                                  exception=lexer.LexerError)
+                                  exception=exceptions.LexerError)
 
     def test_bad_lexer_literal_value_with_json_object(self):
         error_message = ('Bad jmespath expression: '
                          'Bad token `{{}`:\n`{{}`\n^')
         self.assert_error_message('`{{}`', error_message,
-                                  exception=lexer.LexerError)
+                                  exception=exceptions.LexerError)
 
 
     def test_bad_unicode_string(self):
         error_message = ('Bad jmespath expression: '
                          'Invalid \\uXXXX escape:\n"\\uAZ12"\n^')
         self.assert_error_message(r'"\uAZ12"', error_message,
-                                  exception=lexer.LexerError)
+                                  exception=exceptions.LexerError)
 
 
 class TestParserWildcards(unittest.TestCase):
@@ -228,7 +223,7 @@ class TestParserWildcards(unittest.TestCase):
 
     def test_escape_sequence_at_end_of_string_not_allowed(self):
         with self.assertRaises(ValueError):
-            parsed = self.parser.parse('foobar\\')
+            self.parser.parse('foobar\\')
 
     def test_wildcard_with_multiselect(self):
         parsed = self.parser.parse('foo.*.{a: a, b: b}')
@@ -251,6 +246,7 @@ class TestParserWildcards(unittest.TestCase):
         self.assertIn('a', match[1])
         self.assertIn('b', match[1])
 
+
 class TestMergedLists(unittest.TestCase):
     def setUp(self):
         self.parser = parser.Parser()
@@ -286,14 +282,27 @@ class TestParserCaching(unittest.TestCase):
         p = parser.Parser()
         compiled = []
         compiled2 = []
-        for i in range(parser.Parser._max_size + 1):
+        for i in range(parser.Parser._MAX_SIZE + 1):
             compiled.append(p.parse('foo%s' % i))
         # Rerun the test and half of these entries should be from the
         # cache but they should still be equal to compiled.
-        for i in range(parser.Parser._max_size + 1):
+        for i in range(parser.Parser._MAX_SIZE + 1):
             compiled2.append(p.parse('foo%s' % i))
         self.assertEqual(len(compiled), len(compiled2))
-        self.assertEqual(compiled, compiled2)
+        self.assertEqual(
+            [expr.parsed for expr in compiled],
+            [expr.parsed for expr in compiled2])
+
+    def test_cache_purge(self):
+        p = parser.Parser()
+        first = p.parse('foo')
+        cached = p.parse('foo')
+        p.purge()
+        second = p.parse('foo')
+        self.assertEqual(first.parsed,
+                         second.parsed)
+        self.assertEqual(first.parsed,
+                         cached.parsed)
 
 
 class TestParserAddsExpressionAttribute(unittest.TestCase):
